"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const crypto = require("crypto");
const path = require("path");
const cli_utils_1 = require("@ionic/cli-utils");
const command_1 = require("@ionic/cli-utils/lib/command");
const fs_1 = require("@ionic/cli-framework/utils/fs");
const klaw = require("klaw");
class DeployManifestCommand extends command_1.Command {
    constructor() {
        super(...arguments);
        this.buildDir = path.resolve(this.env.project.directory, 'www');
    }
    getMetadata() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return {
                name: 'manifest',
                type: 'project',
                summary: 'Generates a manifest file for the deploy service from a built app directory',
                groups: [cli_utils_1.CommandGroup.Hidden],
            };
        });
    }
    run() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const manifest = yield this.getFilesAndSizesAndHashesForGlobPattern();
            yield fs_1.fsWriteFile(path.resolve(this.buildDir, 'pro-manifest.json'), JSON.stringify(manifest), { encoding: 'utf8' });
        });
    }
    getFilesAndSizesAndHashesForGlobPattern() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const items = [];
            return new Promise((resolve, reject) => {
                klaw(this.buildDir)
                    .on('data', item => {
                    if (item.stats.isFile()) {
                        items.push(this.getFileAndSizeAndHashForFile(item.path, item.stats));
                    }
                })
                    .on('error', err => reject(err))
                    .on('end', () => tslib_1.__awaiter(this, void 0, void 0, function* () { return resolve(yield Promise.all(items)); }));
            });
        });
    }
    getFileAndSizeAndHashForFile(file, stat) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const buffer = yield fs_1.fsReadFile(file, { encoding: undefined });
            return {
                file: path.relative(this.buildDir, file),
                size: stat.size,
                hash: this.getHash(buffer),
            };
        });
    }
    getHash(data) {
        const sha256 = crypto.createHash('sha256');
        sha256.update(data);
        return sha256.digest('base64');
    }
}
exports.DeployManifestCommand = DeployManifestCommand;
